
unit main;

{$IFDEF FPC}{$MODE objfpc}{$H+}{$ENDIF}

interface

uses
  mseguiintf,
  msegui,
  msegraphutils,
  mseforms,
  msewindowwidget,
  msetimer,
  mseact,
  mseapplication,
  msedataedits,
  msedropdownlist,
  mseedit,
  mseglob,
  msegraphics,
  mseguiglob,
  mseificomp,
  mseificompglob,
  mseifiglob,
  msemenus,
  msestat,
  msestatfile,
  msestream,
  msetypes,
  sysutils;

type
  tmainfo = class(tmainform)
    xrenderwidget: twindowwidget;
    ttimer1: ttimer;
    edit_nbr_circles: trealspinedit;
    edit_timer_interval: trealspinedit;
    procedure clientpaintexe(const sender: tcustomwindowwidget; const aupdaterect: rectty);
    procedure ttimer1ontimer(const sender: TObject);
    procedure tmainfooncreate(const sender: TObject);
    procedure edit_nbr_circles_onsetintvalue(const sender: TObject; var avalue: Integer; var accept: Boolean);
    procedure edit_timer_interval_onsetintvalue(const sender: TObject; var avalue: Integer; var accept: Boolean);
  private
    angle_incr: double;
    nbr_circles: integer;
  end;

var
  mainfo: tmainfo;

implementation

uses
  main_mfm,
  mx,
  mxlib,
  mxutil,
  cairo,
  mcairoxlib;

{
  https://krazydad.com/tutorials/circles_js/
  https://krazydad.com/tutorials/circles_js/showexample.php?ex=animated_spiral
}

const
  angle_incr_ini = PI / 90;
  angle_incr_var = PI / 3600;

procedure tmainfo.clientpaintexe(const sender: tcustomwindowwidget; const aupdaterect: rectty);
const
  sm_rad = 2.0;
var
  scr: integer;
  vis: PVisual;
  sf: pcairo_surface_t;
  ct: pcairo_t;
  widgetwidth, widgetheight: integer;

  x, y: double;
  cx, cy, angle: double;
  outer_rad: double;
  ratio: double;
  spiral_rad: double;

  i: integer;
begin
  widgetwidth := sender.viewport.cx;
  widgetheight := sender.viewport.cy;

  scr := XDefaultScreen(msedisplay);
  vis := XDefaultVisual(msedisplay, scr);
  {
  XClearWindow(msedisplay, sender.clientwinid);
  }
  sf := cairo_xlib_surface_create(msedisplay, sender.clientwinid, vis, widgetwidth, widgetheight);
  ct := cairo_create(sf);

  cairo_set_source_rgb(ct, 1.0, 1.0, 1.0);
  cairo_paint(ct);

  cairo_set_source_rgb(ct, 0.0, 0.0, 0.0);

  cx := widgetwidth / 2;
  cy := widgetheight / 2;

  outer_rad := widgetwidth * 0.45;

  for i := 1 to nbr_circles do
  begin
    ratio := i / nbr_circles;
    spiral_rad := ratio * outer_rad;

    angle := i * angle_incr;

    x := cx + cos(angle) * spiral_rad;
    y := cy + sin(angle) * spiral_rad;

    cairo_arc(ct, x, y, sm_rad, 0, 2 * PI);
    cairo_fill(ct);
  end;

  cairo_destroy(ct);
  cairo_surface_destroy(sf);
end;

procedure tmainfo.ttimer1ontimer(const sender: TObject);
begin
  angle_incr := angle_incr + angle_incr_var;
  xrenderwidget.invalidate;
end;

procedure tmainfo.tmainfooncreate(const sender: TObject);
begin
  angle_incr := angle_incr_ini;
  nbr_circles := edit_nbr_circles.asinteger; { http://www.freepascal.ru/forum/viewtopic.php?p=75095#p75095 }
end;

procedure tmainfo.edit_nbr_circles_onsetintvalue(const sender: TObject; var avalue: Integer; var accept: Boolean);
begin
  nbr_circles := avalue;
end;

procedure tmainfo.edit_timer_interval_onsetintvalue(const sender: TObject; var avalue: Integer; var accept: Boolean);
begin
  ttimer1.interval := avalue;
end;

end.
